# coding: utf-8
from __future__ import unicode_literals
import os
from os import path

from fabric.contrib import files
from fabric.decorators import task
from fabric.operations import run, local

__all__ = ['rm_by_pattern']

KERNEL_NAME = os.uname()[0].lower()


def ensure_dir(dir, in_local=False):
    """确保指定的dir被创建"""

    if in_local:
        if not path.isdir(dir):
            local("mkdir -p " + dir)
    elif not files.exists(dir):
        run("mkdir -p " + dir)


def remove_dir(dir, in_local=False):
    """删除指定的文件夹"""

    if in_local:
        if not path.isdir(dir):
            local("rm -r " + dir)
    elif not files.exists(dir):
        run("rm -r " + dir)


@task
def rm_by_pattern(directory, pattern, in_local=False):
    """
    删除指定格式的文件

    参数:
        directory: 目录
        pattern: 格式
        in_local: 在本地执行（默认）

    示例:
        fab fs.rm_by_pattern:.,.pyc,True
    """

    if in_local:
        local('find %s |grep %s | xargs rm -rf' % (directory, pattern))
    else:
        run('find %s |grep %s | xargs rm -rf' % (directory, pattern))


def inplace_render(filename, params):
    for key, value in params.items():
        files.sed(filename, '\$\{%s\}' % key, value)



# coding: utf-8
from __future__ import unicode_literals

from fabric.api import run, env, task
from fabric.context_managers import settings

from essay.tasks import config

__all__ = ['sync']


@task
def sync(*packages):
    """从http://pypi.python.org同步包

    用法:
        fab pypi.sync:django==1.3,tornado
    """

    config.check('PYPI_HOST',
                 'PYPI_USER',
                 'PYPI_ROOT')

    with settings(host_string=env.PYPI_HOST, user=env.PYPI_USER):
        cmd = ["pip", "-q", "install", "--no-deps", "-i", "https://pypi.python.org/simple",
               "-d", env.PYPI_ROOT,
               ' '.join(packages)]

        run(" ".join(cmd))



# coding: utf-8
from __future__ import unicode_literals, print_function
from os import path

from fabric.api import task
from fabric.state import env
from fabric.contrib import files


def check(*properties):
    def _check(_property):
        if not hasattr(env, _property):
            msg = 'env has not attribute [{}]'.format(_property)
            print(msg)
            raise Exception(msg)

    for property in properties:
        _check(property)


@task(default=True)
def upload_conf(**context):
    if hasattr(env, 'LOCAL_SERVER_CONF'):
        for local_conf, server_conf in env.LOCAL_SERVER_CONF:
            template_dir, filename = path.dirname(local_conf), path.basename(local_conf)
            venv_dir = env.CURRENT_VIRTUAL_ENV_DIR
            destination = path.join(venv_dir, server_conf)

            files.upload_template(filename, destination, context=context, use_jinja=True, template_dir=template_dir)
    else:
        print('no local conf to upload')



# coding:utf-8
from __future__ import unicode_literals

from fabric.api import run
from fabric.context_managers import settings, hide
from fabric.decorators import task


@task
def kill_by_name(name):
    """
    停止指定特征的进程
    """

    with settings(warn_only=True):
        run("ps aux | grep '%s' | grep -v 'grep' | awk '{print $2}' | xargs kill -9" % name)


@task
def top():
    """
    查看系统负载
    """

    run("top -b | head -n 1")


@task
def ps_by_venv(venv_dir):
    """"
    查看指定虚拟环境的进程CPU占用
    """

    with hide('status', 'running', 'stderr'):
        run("""ps aux | grep -v grep | grep -v supervisor | grep %s | awk '{print $3, "|", $4}'""" % venv_dir)



# coding: utf-8
import os
from fabric.state import env


def install(args):
    cmd = 'pip install -i=%s %s' % (env.PYPI_INDEX, ' '.join(args))
    os.system(cmd)


def default(args):
    cmd = 'pip ' + ' '.join(args)
    os.system(cmd)



# coding: utf-8
from __future__ import unicode_literals

from importlib import import_module


def import_by_path(dotted_path):
    """
    根据路径动态引入模块属性
    """
    try:
        module_path, class_name = dotted_path.rsplit('.', 1)
    except ValueError:
        raise Exception('%s不是一个有效的模块路径' % module_path)

    try:
        module = import_module(module_path)
    except ImportError as e:
        raise Exception('模块引入错误: "%s"' % e)

    try:
        attr = getattr(module, class_name)
    except AttributeError as e:
        raise Exception('模块引入错误: "%s"' % e)

    return attr



# coding: utf-8
from __future__ import unicode_literals, print_function

import logging

from fabric.state import env
from fabric.context_managers import lcd, cd
from fabric.operations import local, run

logger = logging.getLogger(__name__)


def command(cmd, in_local=False, git_path=None):
    print(cmd, '###')

    if in_local:
        if git_path:
            with lcd(git_path):
                return local(cmd)
        else:
            return local(cmd)
    else:
        if git_path:
            with cd(git_path):
                return run(cmd)
        else:
            return run(cmd)


def clone(project_name, in_local=False, git_path=None):
    """
        把项目clone到本地
    """

    if env.GIT_SERVER.startswith('http'):
        cmd = 'git clone %s/%s' % (env.GIT_SERVER, project_name)
    else:
        cmd = 'git clone git@%s:%s' % (env.GIT_SERVER, project_name)
    command(cmd, in_local, git_path)


def reset(in_local=False, git_path=None):
    """
        把项目lone到本地
    """
    cmd = 'git reset --hard'
    command(cmd, in_local, git_path)


def push(branch=None, in_local=False, git_path=None):
    """
        把项目lone到本地
    """
    cmd = 'git push'
    if branch:
        cmd += ' origin ' + branch
    command(cmd, in_local, git_path)


def pull(in_local=False, git_path=None):
    """
        把项目lone到本地
    """
    cmd = 'git pull'

    command(cmd, in_local, git_path)


def add(files=None, add_all=False, in_local=False, git_path=None):
    if not files and not add_all:
        raise Exception('无效参数')

    if add_all:
        cmd = 'git add .'
    else:
        if not isinstance(files, (tuple, list)):
            files = [files]
        cmd = 'git add ' + ' '.join(files)
    command(cmd, in_local, git_path)


def commit(msg, in_local=False, git_path=None):
    """
        把项目lone到本地
    """
    cmd = 'git commit -a -m "%s"' % msg
    command(cmd, in_local, git_path)


def checkout(commit_or_branch, in_local=False, git_path=None):
    """
        根据commit回滚代码或者获取分支的所有代码

        commit据有优先权
    """

    cmd = 'git reset --hard && git fetch && git checkout %s && git pull && git submodule update --init --recursive' % commit_or_branch
    command(cmd, in_local, git_path)


def get_version(in_local=False, git_path=None):
    cmd = "git rev-parse HEAD"
    return command(cmd, in_local, git_path)



# coding: utf-8
from __future__ import unicode_literals

import pip
try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse

from fabric.api import run, env
from fabric.context_managers import settings
from fabric.contrib.files import exists
from fabric.decorators import task

__all__ = ['install']

PIP_VERSION = int(pip.__version__.split('.')[0])


def is_virtualenv_installed_in_system():
    """
    检查virtualenv是否在系统目录安装
    """

    with settings(warn_only=True):
        return 'no virtualenv' not in run('which virtualenv') or \
            'which' not in run('which virtualenv')


def is_virtualenv_installed_in_user():
    """
    检查virtualenv是否在系统目录安装
    """

    with settings(warn_only=True):
        return exists('~/.local/bin/virtualenv')


def is_virtualenv_installed():
    """
    检查virtualenv是否已安装
    """

    return is_virtualenv_installed_in_system() or is_virtualenv_installed_in_user()


def is_installed(package):
    """检查Python包是否被安装

    注意：只能在虚拟Python环境中执行
    """

    if 'CURRENT_VIRTUAL_ENV_DIR' not in env:
        raise Exception('只可以在虚拟环境安装Python包')
    venv_dir = env.CURRENT_VIRTUAL_ENV_DIR

    with settings(warn_only=True):
        res = run('%(venv_dir)s/bin/pip freeze' % locals())
    packages = [line.split('==')[0].lower() for line in res.splitlines()]

    return package.lower() in packages


@task
def install(package_name, version=None, private=True, user_mode=True):
    """
    用Pip安装Python包

    参数:
        package: 包名，可以指定版本，如Fabric==1.4.3
        private: 利用私有PYPI安装
        user_mode: 安装在用户目录

    注意：只能在虚拟Python环境中执行
    """

    if 'CURRENT_VIRTUAL_ENV_DIR' not in env:
        raise Exception('只可以在虚拟环境安装Python包')

    venv_dir = env.CURRENT_VIRTUAL_ENV_DIR

    options = []

    if hasattr(env, 'HTTP_PROXY'):
        options.append('--proxy {}'.format(env.HTTP_PROXY))

    if private:
        options.append('-i {}'.format(env.PYPI_INDEX))

    if PIP_VERSION >= 7:
        host = urlparse(env.PYPI_INDEX).netloc.split(':')[0]
        options.append('--trusted-host {}'.format(host))

    options_str = ' '.join(options)
    if version:
        package_name += '==' + version

    command = '%(venv_dir)s/bin/pip install %(options_str)s %(package_name)s' % locals()

    run(command)


def ensure(package, private=True, user_mode=True):
    """检查Python包有没有被安装，如果没有则安装

    注意：只能在虚拟Python环境中执行
    """

    if not is_installed(package):
        install(package, private=private, user_mode=user_mode)


def uninstall(package):
    """卸载Python包

    注意：只能在虚拟Python环境中执行
    """

    if 'CURRENT_VIRTUAL_ENV_DIR' not in env:
        raise Exception('只可以在虚拟环境安装Python包')

    venv_dir = env.CURRENT_VIRTUAL_ENV_DIR
    run("%(venv_dir)s/bin/pip uninstall -y %(package)s" % locals())



#!/usr/bin/env python
# encoding: utf-8
from __future__ import unicode_literals, print_function

import datetime
import os
import re
import requests

from fabric.state import env
from fabric.api import cd, run, task, roles

from essay.tasks import git, config, fs
from fabric.contrib import files

__all__ = ['build', 'get_latest_version', 'get_next_version']


PYPI_VERSION_RE = re.compile(r'(\d+(\.\d+)+)')
A_MARKUP_RE = re.compile(r'<a href="(.*?)"')


@roles('build')  # 默认使用build role
@task(default=True)
def build(name=None, version=None, commit=None, branch=None):
    """
    打包

    参数:
        name: 描述, 如:seo。最后生成project_name-x.x.x.x-seo.tar.gz
        commit: 指定commit版本
        branch: 分支名称
        version: 自定义版本号，如果为None则根据日期生成

    commit和branch必须提供一个, 或者读取配置文件
    """

    if commit:
        check_out = commit
    elif branch:
        check_out = branch
    else:
        check_out = env.DEFAULT_BRANCH

    if not version:
        config.check('PROJECT')
        version = get_next_version(env.PROJECT)

    if name:
        version = '%s-%s' % (version, name)

    project_path = os.path.join(env.BUILD_PATH, env.PROJECT)

    if not files.exists(project_path):
        with(cd(env.BUILD_PATH)):
            git.clone('/'.join([env.PROJECT_OWNER, env.PROJECT]))

    with(cd(project_path)):
        git.checkout(check_out)
        # 在setup打包之前做进一步数据准备工作的hook
        if hasattr(env, 'PRE_BUILD_HOOK'):
            env.PRE_BUILD_HOOK()

        params = {
            'release_time': datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
            'git_version': git.get_version(),
            'version': version,
        }

        fs.inplace_render(os.path.join(project_path, 'setup.py'), params)

        if hasattr(env, 'SETTINGS_BASE_FILE'):
            settings_file_path = os.path.join(project_path, *env.SETTINGS_BASE_FILE.split('/'))
            if files.exists(settings_file_path):
                fs.inplace_render(settings_file_path, params)
        else:
            settings_file_path = os.path.join(project_path, env.PROJECT, 'settings.py')
            if files.exists(settings_file_path):
                fs.inplace_render(settings_file_path, params)

            settings_dir_path = os.path.join(project_path, env.PROJECT, 'settings', '__init__.py')
            if files.exists(settings_dir_path):
                fs.inplace_render(settings_dir_path, params)

        run("python setup.py sdist bdist_wheel upload -r internal")


def get_pypi_version(package, repo_url):
    content = requests.get(repo_url).text
    links = A_MARKUP_RE.findall(content)
    versions = [PYPI_VERSION_RE.search(link).group()
                for link in links if package in link]
    return max(versions, key=lambda v: list(map(int, v.split('.')))) \
        if versions else None


@task
def get_latest_version(package_name=None):
    if not package_name:
        config.check('PROJECT')
        package_name = env.PROJECT
    pypi_version = get_pypi_version(
        package_name,
        env.PYPI_INDEX + '/' + env.PROJECT.replace('_', '-'),
    )
    print('current version:{}'.format(pypi_version) or 'no version found')
    return pypi_version


@task
def get_next_version(package_name=None):
    """计算下一个版本号"""

    if not package_name:
        config.check('PROJECT')
        package_name = env.PROJECT

    now = datetime.datetime.now()
    prefix = '%s.%s.%s' % (str(now.year)[-1], now.month, now.day)

    latest_version = get_latest_version(package_name)
    # 如果该项目没有建立过版本,从1开始
    if not latest_version:
        index = 1
    else:
        last_prefix, last_index = latest_version.rsplit('.', 1)

        if last_prefix != prefix:
            index = 1
        else:
            index = int(last_index) + 1

    version = prefix + '.' + str(index)
    print('next version is: {}'.format(version))

    return version



# coding: utf-8
from __future__ import unicode_literals

from fabric.api import parallel, task
from fabric.state import env

from essay.tasks import virtualenv, supervisor, package, build

__all__ = ['deploy', 'quickdeploy']


@task(default=True)
@parallel(30)
def deploy(version, venv_dir, profile):
    """ 发布指定的版本，会自动安装项目运行所需要的包

        version：build之后的版本
        venv_dir：虚拟环境名称
        profile：profile参数会传递到supervisord.conf中
    """

    if not version:
        version = build.get_latest_version()

    virtualenv.ensure(venv_dir)

    pre_hook = env.DEPLOY_PRE_HOOK
    post_hook = env.DEPLOY_POST_HOOK

    with virtualenv.activate(venv_dir):
        if callable(pre_hook):
            pre_hook(version, venv_dir, profile)
        supervisor.ensure(project=env.PROJECT, profile=profile)
        package.install(env.PROJECT, version)
        supervisor.shutdown()
        supervisor.start()
        if callable(post_hook):
            post_hook(version, venv_dir, profile)


@task(default=True)
def quickdeploy(venv_dir, profile, branch=None):
    """
    快速部署

        $ fab -R yourroles quickdeploy:a,test,master
    """

    deploy_host_string = env.host_string

    build_host = env.roledefs.get('build')
    env.host_string = build_host[0] if isinstance(build_host, list) else build_host
    build.build(branch=branch)

    env.host_string = deploy_host_string
    version = build.get_latest_version()

    deploy(version, venv_dir, profile)



# coding:utf-8

__version__ = '${version}'
__git_version__ = '${git_version}'
__release_time__ = '${release_time}'

import os
from log import init_log
PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))

init_log()



# coding: utf-8
from __future__ import unicode_literals, print_function

from fabric.api import task
from fabric.state import env
from fabric.network import parse_host_string

from essay.utils import import_by_path


__all__ = ['validate']


class Validator(object):
    def __init__(self, venv, role, *args, **kwargs):
        self.venv = venv
        self.role = role
        self.args = args
        self.kwargs = kwargs

    def run(self):
        hosts = self.get_hosts()
        ports = self.get_ports()

        result = True
        for host in hosts:
            for port in ports:
                result = self.validate(host, port)
                if not result:
                    print("验证失败: 主机[{}] 端口[{}]".format(host, port))
        if result:
            print("验证成功")

    def validate(self, host, port):
        """
        验证服务可用性

        子类重载此函数，并返回True或False
        """
        raise NotImplementedError

    def get_hosts(self):
        return [
            parse_host_string(host_string)['host']
            for host_string in env.roledefs[self.role]
        ]

    def get_ports(self):
        port_prefix = env.VENV_PORT_PREFIX_MAP[self.venv]
        return [
            int(str(port_prefix) + str(port_suffix))
            for port_suffix in range(env.PROCESS_COUNT)
        ]


@task
def validate(venv, role='product', *args, **kwargs):
    """
    验证部署是否成功

    参数:
        venv: 虚拟环境名称
        role: 服务器角色

    用法:
        $ fab validate:a,product,[custom_args]

    """
    validator_path = env.VALIDATOR_CLASS
    validator_class = import_by_path(validator_path)

    try:
        validator_instance = validator_class(venv, role, *args, **kwargs)
    except:
        raise Exception("{}不是一个有效的Validator".format(validator_path))

    validator_instance.run()



#!/usr/bin/env python
# encoding: utf-8
from __future__ import unicode_literals, print_function

import random
import string

from fabric.decorators import task

try:
    from string import lowercase
except ImportError:
    from string import ascii_lowercase as lowercase

try:
    from string import uppercase
except ImportError:
    from string import ascii_uppercase as uppercase

__all__ = ['random_str']

KEYS = [lowercase,
        uppercase,
        string.digits,
        string.punctuation]


@task
def random_str(length=10, level=1):
    """
    生成随机字符串

    参数:
        length: 字符串长度
        level: 使用的字符集
            1 -> abcdefghijklmnopqrstuvwxyz
            2 -> abcdefghijklmnopqrstuvwxyz + ABCDEFGHIJKLMNOPQRSTUVWXYZ
            3 -> abcdefghijklmnopqrstuvwxyz + ABCDEFGHIJKLMNOPQRSTUVWXYZ + 0123456789
            4 -> abcdefghijklmnopqrstuvwxyz + ABCDEFGHIJKLMNOPQRSTUVWXYZ + 0123456789 + !"#$%&'()*+,-./:;<=>?@[\]^_`{|}~
    """

    if length < 1 or 4 < level < 1:
        raise ValueError('无效参数')

    level = int(level) + 1
    keys = ''.join(KEYS[:level])

    result = ''.join([random.choice(keys) for i in range(length)])

    print(result)

    return result



# coding: utf-8

"""
创建新工程结构
"""

import os
import string
import logging
from os import path

from fabric.api import lcd
from fabric.operations import prompt

from essay import settings
from essay.tasks import fs
from essay.tasks import git

logger = logging.getLogger(__name__)


def create_project(project, template='default'):
    """创建本地工程"""
    init_project(project, template)
    with lcd(project):
        git.command('git init', in_local=True)
        git.add(add_all=True, in_local=True)
        git.commit(u'初始化工程结构', in_local=True)
        repos = prompt('请输入Git仓库地址:')
        if repos:
            git.command('git remote add origin %s' % repos, in_local=True)
            git.command('git push -u origin master', in_local=True)


def init_project(project, template='default'):
    """初始化本地项目

    此方法不需要连接git服务器
    """
    if project is None:
        project_dir = path.abspath('.') 
        template = 'init'
        project = ''
        params = {
            'project_name': project
        }
    else:
        project_dir = path.abspath(project)
        fs.ensure_dir(project, in_local=True)

        params = {
            'project_name': project
        }

    build_structure(project, project_dir, params, template)


def build_structure(project, dst, params, template='default'):
    """
        拷贝工程打包及fab文件到工程
    """
    dst = dst.rstrip('/')

    template_dir = path.join(settings.PROJECT_ROOT, 'templates', template)
    for root, dirs, files in os.walk(template_dir):
        for name in files:
            if name.endswith('.tpl'):
                src = path.join(root, name)
                dst_filename = src.replace(template_dir, dst).rstrip('.tpl').replace('__project__', project)
                dst_dir = os.path.dirname(dst_filename)

                fs.ensure_dir(dst_dir, in_local=True)

                content = open(src).read().decode('utf-8')
                if not name.endswith('.conf.tpl'):
                    content = string.Template(content).safe_substitute(**params)

                open(dst_filename, 'w').write(content.encode('utf-8'))



# coding:utf-8
from __future__ import unicode_literals

from os import path

from fabric.api import run, settings
from fabric.colors import green, red
from fabric.context_managers import cd
from fabric.contrib import files
from fabric.decorators import task
from fabric.state import env

from essay.tasks import config, util, virtualenv, package

__all__ = ['start_process', 'stop_process', 'restart_process', 'reload']


def ensure(**context):
    if 'CURRENT_VIRTUAL_ENV_DIR' not in env:
        raise Exception('只可以在虚拟环境安装Python包')
    venv_dir = env.CURRENT_VIRTUAL_ENV_DIR

    package.ensure('supervisor')

    context.setdefault('run_root', venv_dir)
    context.setdefault('username', util.random_str(10))
    context.setdefault('password', util.random_str(20, True))
    context.setdefault('process_count', 2)
    context.setdefault('venv_dir', venv_dir)
    context.setdefault('virtualenv_name', venv_dir[-1:])
    if 'VENV_PORT_PREFIX_MAP' in env and isinstance(env.VENV_PORT_PREFIX_MAP, dict):
        try:
            context.setdefault('port', env.VENV_PORT_PREFIX_MAP[venv_dir[-1:]])
        except KeyError:
            raise Exception('你的端口配置VENV_DIR_PORT_MAP中key[%s]不存在!' % venv_dir[-1:])
    if 'PROCESS_COUNT' in env:
        context['process_count'] = env.PROCESS_COUNT
    config.check('SUPERVISOR_CONF_TEMPLATE')
    config_template = env.SUPERVISOR_CONF_TEMPLATE
    destination = path.join(venv_dir, 'etc', 'supervisord.conf')

    template_dir, filename = path.dirname(config_template), path.basename(config_template)

    files.upload_template(filename, destination, context=context, use_jinja=True, template_dir=template_dir)


def _supervisor_command(command, venv_dir=None):
    if venv_dir:
        with virtualenv.activate(venv_dir):
            _supervisor_command(command)

    if 'CURRENT_VIRTUAL_ENV_DIR' not in env:
        raise Exception('只可以在虚拟环境安装Python包')

    venv_dir = env.CURRENT_VIRTUAL_ENV_DIR

    # 停止supervisor管理的进程
    with settings(warn_only=True), cd(venv_dir):
        run('bin/supervisorctl -c etc/supervisord.conf ' + command)


@task
def start(venv_dir=None, retry=0, retry_interval=2, max_retries=3):
    """
    重启指定虚拟环境的supervisor
    venv_dir 指定虚拟环境地址
    retry 当前重试次数
    retry_interval 多少秒后开始重试
    max_retries 最大重试次数
    """

    if retry > max_retries:
        print(red('start supervisord FAIL!'))
        return

    if venv_dir:
        with virtualenv.activate(venv_dir):
            start()

    if 'CURRENT_VIRTUAL_ENV_DIR' not in env:
        raise Exception('只可以在虚拟环境安装Python包')

    venv_dir = env.CURRENT_VIRTUAL_ENV_DIR

    with settings(warn_only=True), cd(venv_dir):
        # 停止supervisor管理的进程
        result = run('bin/supervisord -c etc/supervisord.conf ')
        if result:
            retry += 1
            print(green('start supervisord fail, retry[{}]'.format(retry)))
            start(retry=retry, retry_interval=retry_interval, max_retries=max_retries)


@task
def shutdown(venv_dir=None):
    """重启指定虚拟环境的supervisor"""

    _supervisor_command('shutdown', venv_dir)


@task
def reload(venv_dir=None):
    """重启指定虚拟环境的supervisor"""

    _supervisor_command('reload', venv_dir)


@task
def start_process(name, venv_dir=None):
    """
    启动进程
    """

    _supervisor_command(' start ' + name, venv_dir)


@task
def stop_process(name, venv_dir=None):
    """
    关闭进程
    """

    _supervisor_command(' stop ' + name, venv_dir)


@task
def restart_process(name, venv_dir=None):
    """
    重启进程
    """

    _supervisor_command(' restart ' + name, venv_dir)



# coding:utf-8
from __future__ import unicode_literals

import posixpath
from contextlib import contextmanager
from os import path

from fabric.api import run, prompt
from fabric.contrib.files import exists
from fabric.context_managers import prefix
from fabric.state import env

from essay.tasks import process, package, fs

__all__ = []


def ensure(venv_dir, sub_dirs=None, user_mode=True):
    """
    确保虚拟环境存在

    ::
    .. _virtual environment: http://www.virtualenv.org/
    """

    if not venv_dir.startswith('/'):
        if 'VIRTUALENV_PREFIX' in env:
            venv_dir = path.join(env.VIRTUALENV_PREFIX, venv_dir)
        else:
            user_home = run('USER_HOME=$(eval echo ~${SUDO_USER}) && echo ${USER_HOME}')
            venv_dir = path.join(user_home, 'w', venv_dir)

    if is_virtualenv(venv_dir):
        return

    if env.VIRTUALENV_BIN:
        virtualenv_bin = env.VIRTUALENV_BIN
        command = '%(virtualenv_bin)s "%(venv_dir)s"' % locals()
    else:
        if package.is_virtualenv_installed_in_system():
            virtualenv_bin = 'virtualenv'
        else:
            virtualenv_bin = '~/.local/bin/virtualenv'

        command = '%(virtualenv_bin)s --quiet "%(venv_dir)s"' % locals()

    run(command)

    if not sub_dirs:
        sub_dirs = ['logs', 'etc', 'tmp']

    if 'VIRTUALENV_SUB_DIRS' in env:
        sub_dirs = list(set(sub_dirs + env.VIRTUALENV_SUB_DIRS))

    for sub_dir in sub_dirs:
        fs.ensure_dir(path.join(venv_dir, sub_dir))


@contextmanager
def activate(venv_dir, local=False):
    """
    用来启用VirtualEnv的上下文管理器

    ::
        with virtualenv('/path/to/virtualenv'):
            run('python -V')

    .. _virtual environment: http://www.virtualenv.org/
    """

    if not venv_dir.startswith('/'):
        if 'VIRTUALENV_PREFIX' in env:
            venv_dir = path.join(env.VIRTUALENV_PREFIX, venv_dir)
        else:
            user_home = run('USER_HOME=$(eval echo ~${SUDO_USER}) && echo ${USER_HOME}')
            venv_dir = path.join(user_home, 'w', venv_dir)

    if not is_virtualenv(venv_dir):
        raise Exception('无效虚拟环境: %s' % venv_dir)

    join = path.join if local else posixpath.join
    with prefix('. "%s"' % join(venv_dir, 'bin', 'activate')):
        env.CURRENT_VIRTUAL_ENV_DIR = venv_dir
        yield
        # del env['CURRENT_VIRTUAL_ENV_DIR']


def is_virtualenv(venv_dir):
    """判断指定的虚拟环境是否正确"""
    return exists(path.join(venv_dir, 'bin', 'activate'))


def remove(venv_dir):
    """删除指定的虚拟环境"""

    answer = prompt(u"确定删除虚拟环境:%s  (y/n)?" % venv_dir)
    if answer.lower() in ['y', 'yes']:
        if is_virtualenv(venv_dir):
            process.kill_by_name(path.join(venv_dir, 'bin'))
            run('rm -rf ' + venv_dir)



# coding: utf-8
from __future__ import unicode_literals

from setuptools import setup, find_packages

from essay import VERSION

setup(
    name='essay',
    version=VERSION,
    description='持续部署工具',
    long_description='',
    author='SohuTech',
    author_email='thefivefire@gmail.com',
    url='http://github.com/SohuTech/essay',
    packages=find_packages(exclude=['*.pyc', 'fabfile']),
    include_package_data=True,
    install_requires=[
        'Fabric3',
        'Jinja2',
        'requests',
        ],
    entry_points={
        'console_scripts': [
            'es = essay.main:main',
            'ep = essay.main:pip_main',
        ]
    },
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: BSD License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)



# coding: utf-8
from __future__ import unicode_literals

import sys
import logging.config
from os import path


def init_log(log_dir=None):
    if log_dir and not path.exists(log_dir):
        msg = '指定路径不存在:%s' % log_dir
        sys.stdout(msg)
        log_dir = None

    config = {
        'version': 1,
        'disable_existing_loggers': False,
        'formatters': {
            'default': {
                'format': '%(levelname)s %(asctime)s %(module)s:%(funcName)s:%(lineno)d %(message)s'
            },
            'simple': {
                'format': '%(level)s %(message)s'
            }
        },
        'handlers': {
            'console': {
                'level': 'DEBUG',
                'class': 'logging.StreamHandler',
                'formatter': 'default',
            },
        },
        'loggers': {
            '': {
                'handlers': ['console'],
                'level': 'INFO',
                'propagate': True,
            },
        }
    }

    if log_dir:
        config['handles']['file'] = {
            'level': 'DEBUG',
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': path.join(log_dir, 'essay.log'),
            'maxBytes': 1024 * 1024 * 50,
            'backupCount': 5,
            'formatter': 'default',
        }
        config['loggers']['handlers'] = ['console', 'file']

    logging.config.dictConfig(config)



#!/usr/bin/env python
# encoding: utf-8
from __future__ import unicode_literals

from fabric.api import run, env, sudo
from fabric.contrib import files
from fabric.decorators import task

from essay.tasks import config


def _nginx_command(command, nginx_bin=None, nginx_conf=None, use_sudo=False):
    if not nginx_bin:
        config.check('NGINX_BIN')
        nginx_bin = env.NGINX_BIN

    if not nginx_conf:
        config.check('NGINX_CONF')
        nginx_conf = env.NGINX_CONF

    if command == 'start':
        cmd = '%(nginx_bin)s -c %(nginx_conf)s' % locals()
    else:
        cmd = '%(nginx_bin)s -c %(nginx_conf)s -s %(command)s' % locals()

    if use_sudo:
        sudo(cmd)
    else:
        run(cmd)


@task
def stop(nginx_bin=None, nginx_conf=None, use_sudo=False):
    """
    停止Nginx

    参数:
        nginx_bin: nginx可执行文件路径，如果为提供则从env获取。
        nginx_conf: nginx配置文件路径，如果为提供则从env获取。
    """

    _nginx_command('stop', nginx_bin, nginx_conf, use_sudo=use_sudo)


@task
def start(nginx_bin=None, nginx_conf=None, use_sudo=False):
    """
    启动Nginx

    参数:
        nginx_bin: nginx可执行文件路径，如果为提供则从env获取。
        nginx_conf: nginx配置文件路径，如果为提供则从env获取。
    """

    _nginx_command('start', nginx_bin, nginx_conf, use_sudo=use_sudo)


@task
def reload(nginx_bin=None, nginx_conf=None, use_sudo=False):
    """
    重启Nginx

    参数:
        nginx_bin: nginx可执行文件路径，如果为提供则从env获取。
        nginx_conf: nginx配置文件路径，如果为提供则从env获取。
    """
    _nginx_command('reload', nginx_bin, nginx_conf, use_sudo=use_sudo)


@task
def switch(src_pattern, dst_pattern, root=None, nginx_bin=None, nginx_conf=None):
    """
    修改配置文件并重启：源文本,目标文本,[root]（使用root)

    主要用于AB环境的切换，将配置文件中的src_pattern修改为dst_pattern，并重启。

    参数:
        src_pattern: 源模式，如upstreamA
        src_pattern: 目标模式，如upstreamB
        nginx_bin: nginx可执行文件路径，如果为提供则从env获取。
        nginx_conf: nginx配置文件路径，如果为提供则从env获取。
    """

    if not nginx_conf:
        config.check('NGINX_CONF')
        nginx_conf = env.NGINX_CONF

    use_sudo = (root == 'root')
    files.sed(nginx_conf, src_pattern, dst_pattern, use_sudo=use_sudo)
    reload(nginx_bin, nginx_conf, use_sudo=use_sudo)



# coding:utf-8
from __future__ import unicode_literals

import re
import sys
import pip
from optparse import OptionParser
from fabric.state import env

from essay.project import create_project, init_project

usage = """es usage: es create/init [project_name]
    Commands available:
        create: create project with full structure
        pinstall: this command help you install package from our pypi server
        and other PIP support command
    """


def init_options():
    parser = OptionParser(usage=usage)

    parser.add_option("-t", "--template", dest="template", default="default",
                      help="project template:[default],[django]")

    return parser.parse_args()


def main():
    help_text = usage

    if len(sys.argv) == 2 and sys.argv[1] == 'init':
        init_project(None, 'init')
    elif len(sys.argv) >= 2 and sys.argv[1] == 'create':
        options, args = init_options()
        project_name = sys.argv[2]
        if re.match('^[a-zA-Z0-9_]+$', project_name):
            create_project(project_name, options.template)
        else:
            sys.stdout.write('无效工程名: ' + project_name)
    elif len(sys.argv) >= 2 and sys.argv[1] == 'init':
        options, args = init_options()
        project_name = sys.argv[2]
        if re.match('^[a-zA-Z0-9_]+$', project_name):
            init_project(project_name, options.template)
        else:
            sys.stdout.write('无效工程名: ' + project_name)
    elif len(sys.argv) >= 2 and sys.argv[1] == 'pinstall':
        if len(sys.argv) == 2 or sys.argv[2] == '-h':
            sys.stdout.write("es pinstall <package>")
            return

        args = sys.argv[1:]
        args[0] = 'install'
        args.append('-i %s' % env.PYPI_INDEX)
        pip.main(args)
    else:
        if len(sys.argv) == 2 and '-h' in sys.argv:
            sys.stdout.write(help_text)
        pip.main()


if __name__ == '__main__':
    main()
